[{"id":"node_modules/@jenkins-cd/js-modules/js/ModuleSpec.js","source":"var Version = require('./Version');\n\nfunction ModuleSpec(qName) {\n    var qNameTokens = qName.split(\":\");\n    \n    if (qNameTokens.length === 2) {\n        var namespace = qNameTokens[0].trim();\n        var nsTokens = namespace.split(\"/\");\n        var namespaceProvider = undefined;\n        if (nsTokens.length === 2) {\n            namespaceProvider = nsTokens[0].trim();\n            namespace = nsTokens[1].trim();\n            if (namespaceProvider !== 'plugin' && namespaceProvider !== 'core-assets') {\n                console.error('Unsupported module namespace provider \"' + namespaceProvider + '\". Setting to undefined.');\n                namespaceProvider = undefined;\n            }\n        }\n        \n        var npmName = parseNPMName(qNameTokens[1].trim());\n        \n        this.nsProvider = namespaceProvider;\n        this.namespace = namespace;\n        this.moduleName = npmName.name;\n        this.moduleVersion = npmName.version;\n    } else {\n        // The module/bundle is not in a namespace and doesn't\n        // need to be loaded i.e. it will load itself and export.\n        var npmName = parseNPMName(qNameTokens[0].trim());\n\n        this.moduleName = npmName.name;\n        this.moduleVersion = npmName.version;\n    }\n\n    // Attach version compatibility info\n    var versions = [];\n    \n    if (this.moduleVersion) {\n        var moduleVersionTokens = this.moduleVersion.split(/[,|]+/);\n\n        for (var i = 0; i < moduleVersionTokens.length; i++) {\n            var moduleVersionToken = moduleVersionTokens[i].trim();\n            var parsedVersion = new Version(moduleVersionToken);\n            versions.push(parsedVersion);\n        }\n    }\n    \n    this.moduleCompatVersions = versions;\n}\n\nModuleSpec.prototype.getLoadBundleVersion = function() {\n    if (this.moduleCompatVersions.length === 0) {\n        // If no versions were specified on the name, then we\n        // just return undefined.\n        return undefined;\n    }\n    // If a version is specified, we use the first \"specific\" version\n    // e.g. \"1.1.2\" is specific while \"1.1.x\" and \"any\" are not.\n    for (var i = 0; i < this.moduleCompatVersions.length; i++) {\n        var version = this.moduleCompatVersions[i];\n        if (version.isSpecific()) {\n            return version;\n        }\n    }\n    \n    // If there's no specific version then we return the first\n    // version in the list.\n    return this.moduleCompatVersions[0];\n};\n\nModuleSpec.prototype.getLoadBundleName = function() {\n    var version = this.getLoadBundleVersion();\n    if (version) {\n        return this.moduleName + '@' + version.asBaseVersionString();\n    } else {\n        return this.moduleName;\n    }\n};\n\nModuleSpec.prototype.importAs = function() {\n    if (this.moduleName.charAt(0) === '.') {\n        return this.moduleName;\n    }\n    \n    var version = this.getLoadBundleVersion();\n    var importName = normalizePackageName(this.moduleName);\n    var importNS = this.namespace;\n    \n    if (!importNS) {\n        importNS = importName;\n    }\n    \n    var importAs = importNS + ':' + importName;\n    if (this.nsProvider) {\n        importAs = this.nsProvider + '/' + importAs;\n    }\n    if (version) {\n        importAs += '@' + version.asBaseVersionString();\n    }\n    \n    return importAs;\n};\n\nModuleSpec.prototype.getLoadBundleFileNamePrefix = function() {\n    var version = this.getLoadBundleVersion();\n    var normalizedName = normalizePackageName(this.moduleName);\n    if (version) {\n        // If a version was specified then we only do the script load if a\n        // specific version was provided i.e. loading does not get triggered\n        // by imports that specify non-specific version numbers e.g. \"any\"\n        // or \"1.2.x\". A specific version number would be e.g. \"1.2.3\" i.e.\n        // fully qualified. When loading is not triggered, the import is depending\n        // on another import (with a specific version) or on a bundle do an\n        // export of an internal dependency i.e. on another bundle \"providing\"\n        // the module be exporting it.\n        if (version.isSpecific()) {\n            return normalizedName + '-' + version.asLoadVersionString();\n        } else {\n            return undefined;\n        }\n    } else {\n        return normalizedName;\n    }\n};\n\n/**\n * Normalize an NPM package name by removing all non alpha numerics and replacing\n * with hyphens.\n * @param packageName The NPM package name.\n * @returns The normalized NPM package name.\n */\nfunction normalizePackageName(packageName) {\n    packageName = packageName.replace(/[^\\w.]/g, \"-\"); // replace non alphanumerics.\n    if (packageName.charAt(0) === '-') {\n        packageName = packageName.substring(1);\n    }\n    return packageName;\n};\n\nfunction parseNPMName(resourceName) {\n    if (resourceName.length > 1) {\n        var npmName = {};\n        var orgSlashIndex = resourceName.indexOf('/');\n        \n        if (resourceName.charAt(0) === '@' && orgSlashIndex > 0) {\n            // It's an NPM org package. Strip off the org and package\n            // and add it to the name. We'll get the rest then and parse\n            // that.\n            npmName.name = resourceName.substring(0, orgSlashIndex + 1);\n            \n            // Remove the org part from the name and continue parsing.\n            resourceName = resourceName.substring(orgSlashIndex + 1);\n        } else {\n            // Initialise it so we can append to it below.\n            npmName.name = '';\n        }\n        \n        var versionIndex = resourceName.indexOf('@');\n        if (versionIndex > 0) {\n            npmName.name += resourceName.substring(0, versionIndex);\n            npmName.version = resourceName.substring(versionIndex + 1);\n        } else {\n            npmName.name += resourceName;\n        }\n        \n        return npmName;\n    } else {\n        return {\n            name: resourceName\n        }\n    }\n}\n\nmodule.exports = ModuleSpec;","deps":{"./Version":"node_modules/@jenkins-cd/js-modules/js/Version.js"}},{"id":"node_modules/@jenkins-cd/js-modules/js/Version.js","source":"function Version(version) {\n    this.raw = version;\n    \n    // The version string must start with a digit.\n    // It's not an error for it not to start with a number e.g. it can\n    // be \"any\" and we may introduce other aliases.\n    if (!version || version.length === 0 || isNaN(version.charAt(0))) {\n        return;\n    }\n    \n    function normalizeToken(string) {\n        // remove anything that's not a digit, a dot or an x.\n        var normalized = string.replace(/[^\\d]/g, '');\n        if (normalized === '') {\n            return undefined;\n        }\n        return normalized;\n    }\n    \n    var versionTokens = version.split('.');\n    \n    this.prerelease = undefined;\n    \n    var patchAndPrerelease = '';\n    for (var i = 2; i < versionTokens.length; i++) {\n        if (patchAndPrerelease.length > 0) {\n            patchAndPrerelease += '.';\n        }\n        patchAndPrerelease += versionTokens[i];\n        \n        var separatorIdx = patchAndPrerelease.indexOf('-');\n        if (separatorIdx !== -1) {\n            this.patch = normalizeToken(patchAndPrerelease.substring(0, separatorIdx));\n            this.prerelease = patchAndPrerelease.substring(separatorIdx + 1);\n        } else {\n            this.patch = normalizeToken(patchAndPrerelease);\n        }\n    }\n    \n    if (versionTokens.length >= 2) {\n        this.minor = normalizeToken(versionTokens[1]);\n    }\n    if (versionTokens.length >= 1) {\n        this.major = normalizeToken(versionTokens[0]);\n    }    \n}\n\nVersion.prototype.isSpecific = function() {\n    return (this.major !== undefined && this.minor !== undefined && this.patch !== undefined);\n};\n\n/**\n * Get the \"base\" version string for this version number.\n * <p/>\n * This string should only ever include the major, minor and patch tokens\n * of the version i.e. should never include a prerelease tag.\n * @returns {string}\n */\nVersion.prototype.asBaseVersionString = function(separator) {\n    separator = (separator ? separator : '.');\n    \n    if (!this.major || this.major === 'x') {\n        if (this.raw === 'any') {\n            return this.raw;\n        } else {\n            return undefined;\n        }\n    } else if (!this.minor || this.minor === 'x') {\n        return this.major + separator + 'x';\n    } else if (!this.patch || this.patch === 'x') {\n        return this.major + separator + this.minor + separator + 'x';\n    } else {\n        return this.major + separator + this.minor + separator + this.patch;\n    }\n};\n\n/**\n * Get the load version string for this version number.\n * <p/>\n * This string should only ever include the major, minor and patch tokens\n * of the version i.e. should never include a prerelease tag. This function\n * just calls <code>asBaseVersionString</code>, supplying a hyphen as the\n * separator.\n * @returns {string}\n */\nVersion.prototype.asLoadVersionString = function() {\n    return this.asBaseVersionString('-');\n};\n\nmodule.exports = Version;","deps":{}},{"id":"node_modules/@jenkins-cd/js-modules/js/index.js","source":"var internal = require(\"./internal\");\nvar promise = require(\"./promise\");\nvar ModuleSpec = require('./ModuleSpec');\nvar onRegisterTimeout;\nvar whoami;\n\n/**\n * What's the top level module/bundle name.\n * @param moduleQName The module QName.\n * @returns The module QName, or undefined if unknown.\n */\nexports.whoami = function(moduleQName) {\n    if (moduleQName) {\n        whoami = moduleQName;\n        internal.whoami(whoami);\n    }\n    return whoami;\n};\n\n/**\n * Asynchronously import/require a set of modules.\n *\n * <p>\n * Responsible for triggering the async loading of modules if a given module is not already loaded.\n *\n * @param moduleQNames... A list of module \"qualified\" names, each containing the module name prefixed with the namespace\n * and separated by a colon i.e. \"<namespace>:<moduleName>\" e.g. \"jquery:jquery2\".\n *\n * @return A Promise, allowing async load of all modules. The promise is only fulfilled when all modules are loaded.\n */\nexports.importModule = function() {\n    if (arguments.length === 1) {\n        return internal.importModule(arguments[0], onRegisterTimeout);        \n    }\n    \n    var moduleQNames = [];    \n    for (var i = 0; i < arguments.length; i++) {\n        var argument = arguments[i];\n        if (typeof argument === 'string') {\n            moduleQNames.push(argument);\n        }\n    }\n    \n    if (moduleQNames.length == 0) {\n        throw new Error(\"No module names specified.\");\n    }\n    \n    return promise.make(function (resolve, reject) {\n        var fulfillments = [];\n        \n        function onFulfillment() {\n            if (fulfillments.length === moduleQNames.length) {\n                var modules = [];\n                for (var i = 0; i < fulfillments.length; i++) {\n                    if (fulfillments[i].value) {\n                        modules.push(fulfillments[i].value);\n                    } else {\n                        // don't have everything yet so can't fulfill all.\n                        return;\n                    }\n                }\n                // If we make it here, then we have fulfilled all individual promises, which \n                // means we can now fulfill the top level import promise.\n                resolve(modules);\n            }\n        }        \n        \n        // doRequire for each module\n        for (var i = 0; i < moduleQNames.length; i++) {           \n            function doRequire(moduleQName) {\n                var promise = internal.importModule(moduleQName, onRegisterTimeout);\n                var fulfillment = {\n                    promise: promise,\n                    value: undefined\n                };\n                fulfillments.push(fulfillment);\n                promise\n                    .onFulfilled(function(value) {\n                        fulfillment.value = value;\n                        onFulfillment();\n                    })\n                    .onRejected(function(error) {\n                        reject(error);\n                    });\n            }\n            doRequire(moduleQNames[i]);\n        }\n    }).applyArgsOnFulfill();    \n};\n\n/**\n * Synchronously \"require\" a module that it already loaded/registered.\n *\n * <p>\n * This function will throw an error if the module is not already loaded via an outer call to 'import'\n * (or 'import').\n *\n * @param moduleQName The module \"qualified\" name containing the module name prefixed with the namespace\n * separated by a colon i.e. \"<namespace>:<moduleName>\" e.g. \"jquery:jquery2\".\n *\n * @return The module.\n */\nexports.requireModule = function(moduleQName) {\n    var parsedModuleName = new ModuleSpec(moduleQName);\n    var module = internal.getModule(parsedModuleName);    \n    if (!module) {\n        throw new Error(\"Unable to perform synchronous 'require' for module '\" + moduleQName + \"'. This module is not pre-loaded. \" +\n            \"The module needs to have been asynchronously pre-loaded via an outer call to 'import'.\");\n    }\n    return module.exports;\n};\n\n/**\n * Export a module.\n * \n * @param namespace The namespace in which the module resides, or \"undefined\" if the modules is in\n * the \"global\" module namespace e.g. a Jenkins core bundle.\n * @param moduleName The name of the module. \n * @param module The CommonJS style module, or \"undefined\" if we just want to notify other modules waiting on\n * the loading of this module.\n * @param onError On error callback;\n */\nexports.exportModule = function(namespace, moduleName, module, onError) {\n    try {\n        var moduleQName = (namespace ? namespace + ':' : '') + moduleName;\n        var moduleSpec = new ModuleSpec(moduleQName);\n        var moduleNamespaceObj = internal.getModuleNamespaceObj(moduleSpec);\n        var moduleLoadName = moduleSpec.getLoadBundleName();\n\n        if (moduleNamespaceObj[moduleLoadName]) {\n            if (namespace) {\n                console.debug(\"Jenkins plugin module '\" + namespace + \":\" + moduleName + \"' already registered.\");\n            } else {\n                console.debug(\"Jenkins global module '\" + moduleName + \"' already registered.\");\n            }\n            return;\n        }\n\n        if (!module) {\n            module = {\n                exports: {}\n            };\n        } else if (module.exports === undefined) {\n            module = {\n                exports: module\n            };\n        }\n        moduleNamespaceObj[moduleLoadName] = module;\n\n        // Notify all that the module has been registered. See internal.loadModule also.\n        internal.notifyModuleExported(moduleSpec, module.exports);\n    } catch (e) {\n        console.error(e);\n        if (onError) {\n            onError(e);\n        }\n    }\n};\n\n/**\n * Add a module's CSS to the browser page.\n * \n * <p>\n * The assumption is that the CSS can be accessed at e.g.\n * {@code <rootURL>/plugin/<namespace>/jsmodules/<moduleName>/style.css} i.e.\n * the pluginId acts as the namespace.\n * \n * @param namespace The namespace in which the module resides.\n * @param moduleName The name of the module. \n * @param onError On error callback;\n */\nexports.addModuleCSSToPage = function(namespace, moduleName, onError) {\n    try {\n        internal.addModuleCSSToPage(namespace, moduleName);\n    } catch (e) {\n        console.error(e);\n        if (onError) {\n            onError(e);\n        }\n    }\n};\n\n/**\n * Add a plugin CSS file to the browser page.\n * \n * @param pluginName The Jenkins plugin in which the module resides.\n * @param cssPath The CSS path. \n * @param onError On error callback;\n */\nexports.addPluginCSSToPage = function(pluginName, cssPath, onError) {\n    try {\n        internal.addPluginCSSToPage(pluginName, cssPath);\n    } catch (e) {\n        console.error(e);\n        if (onError) {\n            onError(e);\n        }\n    }\n};\n\n/**\n * Create a js-modules CSS id that can be used to refer to a link element.\n * @param cssPath The CSS path/url.\n * @param namespace Optional namespace.\n */\nexports.toCSSId = function (cssPath, namespace) {\n    return internal.toCSSId(cssPath, namespace);\n};\n\n/**\n * Add CSS file to the browser page.\n * \n * @param cssPath The CSS path. \n * @param onError On error callback;\n */\nexports.addCSSToPage = function(cssPath, onError) {\n    try {\n        if (cssPath.indexOf(internal.getAdjunctURL()) === 0) {\n            internal.addCSSToPage(undefined, cssPath);\n        } else {\n            internal.addCSSToPage(undefined, internal.getRootURL() + '/' + cssPath);\n        }\n    } catch (e) {\n        console.error(e);\n        if (onError) {\n            onError(e);\n        }\n    }\n};\n\n/**\n * Add a javascript &lt;script&gt; element to the document &lt;head&gt;.\n * <p/>\n * Options:\n * <ul>\n *     <li><strong>scriptId</strong>: The script Id to use for the element. If not specified, one will be generated from the scriptSrc.</li>\n *     <li><strong>async</strong>: Asynchronous loading of the script. Default is 'true'.</li>\n *     <li><strong>success</strong>: An optional onload success function for the script element.</li>\n *     <li><strong>error</strong>: An optional onload error function for the script element. This is called if the .js file exists but there's an error evaluating the script. It is NOT called if the .js file doesn't exist (ala 404).</li>\n *     <li><strong>removeElementOnLoad</strong>: Remove the script element after loading the script. Default is 'false'.</li>\n * </ul>\n * \n * @param scriptSrc The script src.\n * @param options Optional script load options object. See above.\n */\nexports.addScript = function(scriptSrc, options) {\n    internal.addScript(scriptSrc, options);\n};\n\n/**\n * Set the module registration timeout i.e. the length of time to wait for a module to load before failing.\n *\n * @param timeout Millisecond duration before onRegister times out. Defaults to 10000 (10s) if not specified.\n */\nexports.setRegisterTimeout = function(timeout) {\n    onRegisterTimeout = timeout;\n}\n\n/**\n * Set the Jenkins root/base URL.\n * \n * @param rootUrl The root/base URL.\n */\nexports.setRootURL = function(rootUrl) {\n    internal.setRootURL(rootUrl);\n};\n\nexports.getRootURL = internal.getRootURL;\nexports.getAdjunctURL = internal.getAdjunctURL;\n\n/**\n * Manually initialise the Jenkins Global.\n * <p>\n * This should only ever be called from a test environment.\n */\nexports.initJenkinsGlobal = function() {\n    internal.initJenkinsGlobal();\n};\n\ninternal.onJenkinsGlobalInit(function(jenkinsCIGlobal) {\n    // For backward compatibility, we need to make some js-modules\n    // functions globally available e.g. to allow legacy code wait for\n    // certain modules to be loaded, as with legacy adjuncts.\n    if (!jenkinsCIGlobal._internal) {\n        // Put the functions on an object called '_internal' as a way\n        // of hinting to people to not use it.\n        jenkinsCIGlobal._internal = {\n            importModule: exports.importModule,\n            addScript: internal.addScript\n        };\n    }\n});","deps":{"./ModuleSpec":"node_modules/@jenkins-cd/js-modules/js/ModuleSpec.js","./internal":"node_modules/@jenkins-cd/js-modules/js/internal.js","./promise":"node_modules/@jenkins-cd/js-modules/js/promise.js"}},{"id":"node_modules/@jenkins-cd/js-modules/js/internal.js","source":"var promise = require(\"./promise\");\nvar ModuleSpec = require('./ModuleSpec');\n\nvar jenkinsCIGlobal;\nvar globalInitListeners = [];\nvar whoami;\n\nexports.whoami = function(moduleQName) {\n    if (moduleQName) {\n        whoami = new ModuleSpec(moduleQName);\n        whoami.nsProvider = getBundleNSProviderFromScriptElement(whoami.namespace, whoami.moduleName);\n    }\n    return whoami;\n};\n\nexports.onJenkinsGlobalInit = function(callback) {\n    globalInitListeners.push(callback);\n};\n\nexports.initJenkinsGlobal = function() {\n    jenkinsCIGlobal = {\n    };\n    if (globalInitListeners) {\n        for (var i = 0; i < globalInitListeners.length; i++) {\n            globalInitListeners[i](jenkinsCIGlobal);\n        }\n    }\n};\n\nexports.clearJenkinsGlobal = function() {    \n    jenkinsCIGlobal = undefined;\n    whoami = undefined;\n};\n\nexports.getJenkins = function() {\n    if (jenkinsCIGlobal) {\n        return jenkinsCIGlobal;\n    }\n    if (window.jenkinsCIGlobal) {\n        jenkinsCIGlobal = window.jenkinsCIGlobal;\n    } else {\n        exports.initJenkinsGlobal();\n        jenkinsCIGlobal.rootURL = getRootURL();\n        window.jenkinsCIGlobal = jenkinsCIGlobal;\n    }   \n    return jenkinsCIGlobal;\n};\n\nexports.getModuleNamespaceObj = function(moduleSpec) {\n    if (moduleSpec.namespace) {\n        return exports.getNamespace(moduleSpec.namespace);\n    } else {\n        return exports.getGlobalModules();\n    }\n};\n\nexports.getNamespace = function(namespaceName) {\n    var namespaces = exports.getNamespaces();\n    var namespace = namespaces[namespaceName];\n    if (!namespace) {\n        namespace = {\n            globalNS: false            \n        };\n        namespaces[namespaceName] = namespace;\n    }\n    return namespace;\n};\n\nexports.importModule = function(moduleQName, onRegisterTimeout) {\n    return promise.make(function (resolve, reject) {\n        var moduleSpec = new ModuleSpec(moduleQName);\n        var module = exports.getModule(moduleSpec);\n\n        if (module) {\n            // module already loaded\n            resolve(module.exports);\n        } else {\n            if (onRegisterTimeout === 0) {\n                if (moduleSpec.namespace) {\n                    throw new Error('Module ' + moduleSpec.namespace + ':' + moduleSpec.moduleName + ' require failure. Async load mode disabled.');\n                } else {\n                    throw new Error('Global module ' + moduleSpec.moduleName + ' require failure. Async load mode disabled.');\n                }\n            }\n\n            // module not loaded. Load async, fulfilling promise once registered\n            exports.loadModule(moduleSpec, onRegisterTimeout)\n                .onFulfilled(function (moduleExports) {\n                    resolve(moduleExports);\n                })\n                .onRejected(function (error) {\n                    reject(error);\n                });\n        }\n    });    \n};\n\nexports.loadModule = function(moduleSpec, onRegisterTimeout) {\n    var module = exports.getModule(moduleSpec);\n    \n    if (module) {\n        // Module already loaded. This prob shouldn't happen.\n        console.log(\"Unexpected call to 'loadModule' for a module (\" + moduleSpec.moduleName + \") that's already loaded.\");\n        return promise.make(function (resolve) {\n            resolve(module.exports);\n        });\n    }\n\n    function waitForRegistration(loadingModule, onRegisterTimeout) {\n        return promise.make(function (resolve, reject) {\n            if (typeof onRegisterTimeout !== \"number\") {\n                onRegisterTimeout = 10000;\n            }\n            \n            var timeoutObj = setTimeout(function () {\n                // Timed out waiting on the module to load and register itself.\n                if (!loadingModule.loaded) {\n                    var moduleSpec = loadingModule.moduleSpec;\n                    var errorDetail;\n                    \n                    if (moduleSpec.namespace) {\n                        errorDetail = \"Timed out waiting on module '\" + moduleSpec.namespace + \":\" + moduleSpec.moduleName + \"' to load.\";\n                    } else {\n                        errorDetail = \"Timed out waiting on module '\" + moduleSpec.moduleName + \"' to load.\";\n                    }                    \n                    console.error('Module load failure: ' + errorDetail);\n\n                    // Call the reject function and tell it we timed out\n                    reject({\n                        reason: 'timeout',\n                        detail: errorDetail\n                    });\n                }\n            }, onRegisterTimeout);\n            \n            loadingModule.waitList.push({\n                resolve: resolve,\n                timeoutObj: timeoutObj\n            });                    \n        });\n    }\n    \n    var moduleNamespaceObj = exports.getModuleNamespaceObj(moduleSpec);\n    var loadModuleName = moduleSpec.getLoadBundleName();\n    var loadVersion = moduleSpec.getLoadBundleVersion();\n    var doScriptLoad = true;\n    \n    if (loadVersion) {\n        // If a version was specified then we only do the script load if a\n        // specific version was provided i.e. loading does not get triggered\n        // by imports that specify non-specific version numbers e.g. \"any\"\n        // or \"1.2.x\". A specific version number would be e.g. \"1.2.3\" i.e.\n        // fully qualified. When loading is not triggered, the import is depending\n        // on another import (with a specific version) or on another bundle to do an\n        // export of an internal dependency i.e. on another bundle \"providing\"\n        // the module by exporting it.\n        doScriptLoad = loadVersion.isSpecific();\n    }\n    \n    var loadingModule = getLoadingModule(moduleNamespaceObj, loadModuleName);\n    if (!loadingModule.waitList) {\n        loadingModule.waitList = [];\n    }\n    loadingModule.moduleSpec = moduleSpec; \n    loadingModule.loaded = false;\n\n    try {\n        return waitForRegistration(loadingModule, onRegisterTimeout);\n    } finally {\n        if (doScriptLoad) {\n            var scriptId = exports.toModuleId(moduleSpec.namespace, loadModuleName) + ':js';\n            var scriptSrc = exports.toModuleSrc(moduleSpec, 'js');\n            var scriptEl = exports.addScript(scriptSrc, {\n                scriptId: scriptId,\n                scriptSrcBase: ''\n            });\n\n            if (scriptEl) {\n                // Set the module spec info on the <script> element. This allows us to resolve the\n                // nsProvider for that bundle after 'whoami' is called for it (as it loads). whoami\n                // is not called with the nsProvider info on it because a given bundle can\n                // potentially be loaded from multiple different ns providers, so we only resole the provider\n                // at load-time i.e. just after a bundle is loaded it calls 'whoami' for itself\n                // and then this module magically works out where it was loaded from (it's nsProvider)\n                // by locating the <script> element and using this information. For a module/bundle, knowing\n                // where it was loaded from is important because it dictates where that module/bundle\n                // should load it dependencies from. For example, the Bootstrap module/bundle depends on the\n                // jQuery bundle. So, if the bootstrap bundle is loaded from the 'core-assets' namespace provider,\n                // then that means the jQuery bundle should also be loaded from the 'core-assets'\n                // namespace provider.\n                // See getBundleNSProviderFromScriptElement.\n                scriptEl.setAttribute('data-jenkins-module-nsProvider', moduleSpec.nsProvider);\n                scriptEl.setAttribute('data-jenkins-module-namespace', moduleSpec.namespace);\n                scriptEl.setAttribute('data-jenkins-module-moduleName', loadModuleName);\n            }\n        }\n    }\n};\n\nexports.addScript = function(scriptSrc, options) {\n    if (!scriptSrc) {\n        console.warn('Call to addScript with undefined \"scriptSrc\" arg.');\n        return undefined;\n    }    \n    \n    var normalizedOptions;\n    \n    // If there's no options object, create it.\n    if (typeof options === 'object') {\n        normalizedOptions = options;\n    } else {\n        normalizedOptions = {};\n    }\n    \n    // May want to transform/map some urls.\n    if (normalizedOptions.scriptSrcMap) {\n        if (typeof normalizedOptions.scriptSrcMap === 'function') {\n            scriptSrc = normalizedOptions.scriptSrcMap(scriptSrc);\n        } else if (Array.isArray(normalizedOptions.scriptSrcMap)) {\n            // it's an array of suffix mappings\n            for (var i = 0; i < normalizedOptions.scriptSrcMap.length; i++) {\n                var mapping = normalizedOptions.scriptSrcMap[i];\n                if (mapping.from && mapping.to) {\n                    if (endsWith(scriptSrc, mapping.from)) {\n                        normalizedOptions.originalScriptSrc = scriptSrc;\n                        scriptSrc = scriptSrc.replace(mapping.from, mapping.to);\n                        break;\n                    }\n                }\n            }\n        }\n    }\n    \n    normalizedOptions.scriptId = getScriptId(scriptSrc, options);\n    \n    // set some default options\n    if (normalizedOptions.async === undefined) {\n        normalizedOptions.async = true;\n    }\n    if (normalizedOptions.scriptSrcBase === undefined) {\n        normalizedOptions.scriptSrcBase = '@root';\n    }\n    \n    if (normalizedOptions.scriptSrcBase === '@root') {\n        normalizedOptions.scriptSrcBase = getRootURL() + '/';\n    } else if (normalizedOptions.scriptSrcBase === '@adjunct') {\n        normalizedOptions.scriptSrcBase = getAdjunctURL() + '/';\n    }\n\n    var document = window.document;\n    var head = exports.getHeadElement();\n    var script = document.getElementById(normalizedOptions.scriptId);\n\n    if (script) {\n        var replaceable = script.getAttribute('data-replaceable');\n        if (replaceable && replaceable === 'true') {\n            // This <script> element is replaceable. In this case, \n            // we remove the existing script element and add a new one of the\n            // same id and with the specified src attribute.\n            // Adding happens below.\n            script.parentNode.removeChild(script);\n        } else {\n            return undefined;\n        }\n    }\n\n    script = createElement('script');\n\n    // Parts of the following onload code were inspired by how the ACE editor does it,\n    // as well as from the follow SO post: http://stackoverflow.com/a/4845802/1166986\n    var onload = function (_, isAborted) {\n        script.setAttribute('data-onload-complete', true);\n        try {\n            if (isAborted) {\n                console.warn('Script load aborted: ' + scriptSrc);\n            } else if (!script.readyState || script.readyState === \"loaded\" || script.readyState === \"complete\") {\n                // If the options contains an onload function, call it.\n                if (typeof normalizedOptions.success === 'function') {\n                    normalizedOptions.success(script);\n                }\n                return;\n            }\n            if (typeof normalizedOptions.error === 'function') {\n                normalizedOptions.error(script, isAborted);\n            }\n        } finally {\n            if (normalizedOptions.removeElementOnLoad) {\n                head.removeChild(script);\n            }\n            // Handle memory leak in IE\n            script = script.onload = script.onreadystatechange = null;\n        }\n    };\n    script.onload = onload; \n    script.onreadystatechange = onload;\n\n    script.setAttribute('id', normalizedOptions.scriptId);\n    script.setAttribute('type', 'text/javascript');\n    script.setAttribute('src', normalizedOptions.scriptSrcBase + scriptSrc);\n    if (normalizedOptions.originalScriptSrc) {\n        script.setAttribute('data-referrer', normalizedOptions.originalScriptSrc);        \n    }\n    if (normalizedOptions.async) {\n        script.setAttribute('async', normalizedOptions.async);\n    }\n    \n    head.appendChild(script);\n    \n    return script;\n};\n\nexports.notifyModuleExported = function(moduleSpec, moduleExports) {\n    var moduleNamespaceObj = exports.getModuleNamespaceObj(moduleSpec);\n    var loadingModule = getLoadingModule(moduleNamespaceObj, moduleSpec.getLoadBundleName());\n    \n    loadingModule.loaded = true;\n    if (loadingModule.waitList) {\n        for (var i = 0; i < loadingModule.waitList.length; i++) {\n            var waiter = loadingModule.waitList[i];\n            clearTimeout(waiter.timeoutObj);\n            waiter.resolve(moduleExports);\n        }\n    }    \n};\n\nexports.addModuleCSSToPage = function(namespace, moduleName) {\n    var moduleSpec = exports.getModuleSpec(namespace + ':' + moduleName);\n    var cssElId = exports.toModuleId(namespace, moduleName) + ':css';\n    var cssPath = exports.toModuleSrc(moduleSpec, 'css');\n    return exports.addCSSToPage(namespace, cssPath, cssElId);\n};\n\nexports.addPluginCSSToPage = function(namespace, cssPath, cssElId) {\n    var cssPath = exports.getPluginPath(namespace) + '/' + cssPath;\n    return exports.addCSSToPage(namespace, cssPath, cssElId);\n};\n\nexports.toCSSId = function (cssPath, namespace) {\n    return 'jenkins-js-module:' + (namespace || 'global' ) + ':css:' + cssPath;\n};\n\nexports.addCSSToPage = function(namespace, cssPath, cssElId) {\n    var document = window.document;\n    \n    if (cssElId === undefined) {\n        cssElId = exports.toCSSId(cssPath, namespace);\n    }\n    \n    var cssEl = document.getElementById(cssElId);\n    \n    if (cssEl) {\n        // already added to page\n        return;\n    }\n\n    var docHead = exports.getHeadElement();\n    cssEl = createElement('link');\n    cssEl.setAttribute('id', cssElId);\n    cssEl.setAttribute('type', 'text/css');\n    cssEl.setAttribute('rel', 'stylesheet');\n    cssEl.setAttribute('href', cssPath);\n    docHead.appendChild(cssEl);\n\n    return cssEl;\n};\n\nexports.getGlobalModules = function() {\n    var jenkinsCIGlobal = exports.getJenkins();\n    if (!jenkinsCIGlobal.globals) {\n        jenkinsCIGlobal.globals = {\n            globalNS: true\n        };\n    }\n    return jenkinsCIGlobal.globals;\n};\n\nexports.getNamespaces = function() {\n    var jenkinsCIGlobal = exports.getJenkins();\n\n    // The namespaces are stored in an object named \"plugins\". This is a legacy from the\n    // time when all modules lived in plugins. By right we'd like to rename this, but\n    // that would cause compatibility issues.\n\n    if (!jenkinsCIGlobal.plugins) {\n        jenkinsCIGlobal.plugins = {\n            __README__: 'This object holds namespaced JS modules/bundles, with the property names representing the module namespace. It\\'s name (\"plugins\") is a legacy thing. Changing it to a better name (e.g. \"namespaces\") would cause compatibility issues.'\n        };\n    }\n    return jenkinsCIGlobal.plugins;\n};\n\nexports.toModuleId = function(namespace, moduleName) {\n    return 'jenkins-js-module:' + (namespace ? namespace + ':' : '') + moduleName;\n};\n\nexports.toModuleSrc = function(moduleSpec, srcType) {\n    var nsProvider = moduleSpec.nsProvider;\n\n    // If a moduleSpec on a module/bundle import doesn't specify a namespace provider\n    // (i.e. is of the form \"a:b\" and not \"core-assets/a:b\"),\n    // then check \"this\" bundles module spec and see if it was imported from a specific\n    // namespace. If it was (e.g. 'core-assets'), then import from that namespace.\n    if (nsProvider === undefined) {\n        nsProvider = thisBundleNamespaceProvider();\n        if (nsProvider === undefined) {\n            nsProvider = 'adjuncts';\n        }\n        // Store the nsProvider back onto the moduleSpec.\n        moduleSpec.nsProvider = nsProvider;\n    }\n\n    var srcPath = undefined;\n    if (srcType === 'js') {\n        srcPath = moduleSpec.getLoadBundleFileNamePrefix() + '.js';\n    } else if (srcType === 'css') {\n        srcPath = moduleSpec.getLoadBundleFileNamePrefix() + '/style.css';\n    } else {\n        throw new Error('Unsupported srcType \"'+ srcType + '\".');\n    }\n\n    if (nsProvider === 'adjuncts') {\n        return exports.getAdjunctJSModulesPath(moduleSpec.namespace) + '/' + srcPath;\n    } else if (nsProvider === 'plugin') {\n        return exports.getPluginJSModulesPath(moduleSpec.namespace) + '/' + srcPath;\n    } else if (nsProvider === 'core-assets') {\n        return exports.getCoreAssetsJSModulesPath(moduleSpec.namespace) + '/' + srcPath;\n    } else {\n        throw new Error('Unsupported namespace provider: ' + nsProvider);\n    }\n};\n\nexports.getAdjunctJSModulesPath = function(namespace) {\n    if (namespace) {\n        return getAdjunctURL() + '/org/jenkins/ui/jsmodules/' + namespace;\n    } else {\n        return getAdjunctURL() + '/org/jenkins/ui/jsmodules';\n    }\n};\n\nexports.getPluginJSModulesPath = function(pluginId) {\n    return exports.getPluginPath(pluginId) + '/jsmodules';\n};\n\nexports.getCoreAssetsJSModulesPath = function(namespace) {\n    return getRootURL() + '/assets/' + namespace + '/jsmodules';\n};\n\nexports.getPluginPath = function(pluginId) {\n    return getRootURL() + '/plugin/' + pluginId;\n};\n\nexports.getHeadElement = function() {\n    var docHead = window.document.getElementsByTagName(\"head\");\n    if (!docHead || docHead.length == 0) {\n        throw new Error('No head element found in document.');\n    }\n    return docHead[0];\n};\n\nexports.setRootURL = function(url) {    \n    if (!jenkinsCIGlobal) {\n        exports.initJenkinsGlobal();\n    }\n    jenkinsCIGlobal.rootURL = url;\n};\n\nexports.getModule = function(moduleSpec) {\n    var namespace = exports.getModuleNamespaceObj(moduleSpec);\n    \n    if (!moduleSpec.moduleVersion) {\n        return namespace[moduleSpec.moduleName];\n    } else {\n        for (var i = 0; i < moduleSpec.moduleCompatVersions.length; i++) {\n            var moduleCompatVersion = moduleSpec.moduleCompatVersions[i];\n            var module = namespace[moduleSpec.moduleName + '@' + moduleCompatVersion.raw];\n            if (module) {\n                return module;\n            }\n        }\n    }\n    \n    return undefined;\n};\n\nexports.getModuleSpec = function(moduleQName) {\n    var moduleSpec = new ModuleSpec(moduleQName);\n    var moduleNamespaceObj = exports.getModuleNamespaceObj(moduleSpec);\n    if (moduleNamespaceObj) {\n        var loading = getLoadingModule(moduleNamespaceObj, moduleSpec.getLoadBundleName());\n        if (loading && loading.moduleSpec) {\n            return loading.moduleSpec;\n        }\n    }\n    return moduleSpec;\n};\n\nfunction getScriptId(scriptSrc, config) {\n    if (typeof config === 'string') {\n        return config;\n    } else if (typeof config === 'object' && config.scriptId) {\n        return config.scriptId;\n    } else {\n        return 'jenkins-script:' + scriptSrc;\n    }    \n}\n\nexports.getRootURL = getRootURL;\nfunction getRootURL() {\n    if (jenkinsCIGlobal && jenkinsCIGlobal.rootURL) {\n        return jenkinsCIGlobal.rootURL;\n    }\n    \n    var docHead = exports.getHeadElement();\n    var rootURL = getAttribute(docHead, \"data-rooturl\");\n\n    if (rootURL === undefined || rootURL === null) {\n        // Backward compatibility - used to use a 'resurl' attribute.\n        rootURL = getAttribute(docHead, \"resurl\");\n        if (rootURL === undefined || rootURL === null) {\n            throw new Error(\"Attribute 'data-rooturl' not defined on the document <head> element.\");\n        }\n    }\n\n    if (jenkinsCIGlobal) {\n        jenkinsCIGlobal.rootURL = rootURL;\n    }\n    \n    return rootURL;\n}\n\nexports.getAdjunctURL = getAdjunctURL;\nfunction getAdjunctURL() {\n    if (jenkinsCIGlobal && jenkinsCIGlobal.adjunctURL) {\n        return jenkinsCIGlobal.adjunctURL;\n    }\n    \n    var docHead = exports.getHeadElement();\n    var adjunctURL = getAttribute(docHead, \"data-adjuncturl\");\n\n    if (adjunctURL === undefined || adjunctURL === null) {\n        // Backward compatibility - older Jenkins do not have the adjunct url on the\n        // <head> element. Lets try getting the resurl (older jenkins) and patching it \n        // to be an adjunct url.\n        adjunctURL = getAttribute(docHead, \"resurl\");\n        if (adjunctURL === undefined || adjunctURL === null) {\n            throw new Error(\"Attribute 'data-adjuncturl' not defined on the document <head> element.\");\n        }\n        // Replace the first occurrence of 'static/' with 'adjuncts/' \n        adjunctURL = adjunctURL.replace('static\\/', 'adjuncts\\/');\n    }\n\n    if (jenkinsCIGlobal) {\n        jenkinsCIGlobal.adjunctURL = adjunctURL;\n    }\n    \n    return adjunctURL;\n}\n\nfunction createElement(name) {\n    var document = window.document;\n    return document.createElement(name);\n}\n\nfunction getAttribute(element, attributeName) {\n    var value = element.getAttribute(attributeName.toLowerCase());\n    \n    if (value) {\n        return value;\n    } else {\n        // try without lowercasing\n        return element.getAttribute(attributeName);\n    }    \n}\n\nfunction getLoadingModule(moduleNamespaceObj, moduleName) {\n    if (!moduleNamespaceObj.loadingModules) {\n        moduleNamespaceObj.loadingModules = {};\n    }\n    if (!moduleNamespaceObj.loadingModules[moduleName]) {\n        moduleNamespaceObj.loadingModules[moduleName] = {};\n    }\n    return moduleNamespaceObj.loadingModules[moduleName];\n}\n\nfunction endsWith(string, suffix) {\n    return (string.indexOf(suffix, string.length - suffix.length) !== -1);\n}\n\nfunction thisBundleNamespaceProvider() {\n    if (whoami !== undefined) {\n        return whoami.nsProvider;\n    }\n    return undefined;\n}\n\nfunction getBundleNSProviderFromScriptElement(namespace, moduleName) {\n    var docHead = exports.getHeadElement();\n    var scripts = docHead.getElementsByTagName(\"script\");\n\n    for (var i = 0; i < scripts.length; i++) {\n        var script = scripts[i];\n        var elNamespace = script.getAttribute('data-jenkins-module-namespace');\n        var elModuleName = script.getAttribute('data-jenkins-module-moduleName');\n\n        if (elNamespace === namespace && elModuleName === moduleName) {\n            return script.getAttribute('data-jenkins-module-nsProvider');\n        }\n    }\n\n    return undefined;\n}","deps":{"./ModuleSpec":"node_modules/@jenkins-cd/js-modules/js/ModuleSpec.js","./promise":"node_modules/@jenkins-cd/js-modules/js/promise.js"}},{"id":"node_modules/@jenkins-cd/js-modules/js/promise.js","source":"exports.make = function(executor) {\n    var thePromise = new APromise();\n    executor.call(thePromise, function(result) {\n        thePromise.resolve(result);\n    }, function(reason) {\n        thePromise.reject(reason);\n    });\n    return thePromise;\n};\n\nfunction APromise() {\n    this.state = 'PENDING';\n    this.whenFulfilled = undefined;\n    this.whenRejected = undefined;\n    this.applyFulfillArgs = false;\n}\n\nAPromise.prototype.applyArgsOnFulfill = function() {\n    this.applyFulfillArgs = true;\n    return this;\n}\n\nAPromise.prototype.resolve = function (result) {\n    this.state = 'FULFILLED';\n    \n    var thePromise = this;\n    function doFulfill(whenFulfilled, result) {\n        if (thePromise.applyFulfillArgs) {\n            whenFulfilled.apply(whenFulfilled, result);\n        } else {\n            whenFulfilled(result);\n        }\n    }\n    \n    if (this.whenFulfilled) {\n        doFulfill(this.whenFulfilled, result);\n    }\n    // redefine \"onFulfilled\" to call immediately\n    this.onFulfilled = function (whenFulfilled) {\n        if (whenFulfilled) {\n            doFulfill(whenFulfilled, result);\n        }\n        return this;\n    }\n};\n\nAPromise.prototype.reject = function (reason) {\n    this.state = 'REJECTED';\n    if (this.whenRejected) {\n        this.whenRejected(reason);\n    }\n    // redefine \"onRejected\" to call immediately\n    this.onRejected = function(whenRejected) {\n        if (whenRejected) {\n            whenRejected(reason);\n        }\n        return this;\n    }\n};\n\nAPromise.prototype.onFulfilled = function(whenFulfilled) {\n    if (!whenFulfilled) {\n        throw new Error('Must provide an \"whenFulfilled\" callback.');\n    }\n    this.whenFulfilled = whenFulfilled;\n    return this;\n};\n\nAPromise.prototype.onRejected = function(whenRejected) {        \n    if (whenRejected) {\n        this.whenRejected = whenRejected;\n    }\n    return this;\n};","deps":{}},{"id":"node_modules/react/react.js","source":"module.exports = require('@jenkins-cd/js-modules').requireModule('react:react@any');","deps":{"@jenkins-cd/js-modules":"node_modules/@jenkins-cd/js-modules/js/index.js"}},{"id":"target/js-bundle-src/react-15-3-2.js","source":"var ___$$$___requiredModuleMappings = [\"react:react@any\"];\n\nvar ___$$$___jsModules = require('@jenkins-cd/js-modules');\n\nif (!window.jenkinsCIGlobal) {\n    window.jenkinsCIGlobal = {};\n}\nif (!window.jenkinsCIGlobal._internal) {\n    window.jenkinsCIGlobal._internal = {};\n}\n\nvar whoami = 'react-15-3-2';\nvar loadTimestamp = Date.now();\n\n___$$$___jsModules.whoami(whoami);\n\n// Clear out the session storage if this is the first\n// bundle to export on the page.\nvar trackKeyPrefix = 'jenkins-cd/js-modules/tracking/';\nif (window.sessionStorage && !jenkinsCIGlobal._internal.bundleLoadIdx) {\n    jenkinsCIGlobal._internal.bundleLoadIdx = 0;\n    var toRemove = [];\n    for (var i = 0; i < sessionStorage.length; i++) {\n        var key = sessionStorage.key(i);\n        if (key.indexOf(trackKeyPrefix) === 0) {\n            toRemove.push(key);\n        }\n    }\n    for (var i = 0; i < toRemove.length; i++) {\n        sessionStorage.removeItem(toRemove[i]);\n    }\n}\njenkinsCIGlobal._internal.bundleLoadIdx++;\n\nif (window.sessionStorage) {\n    sessionStorage.setItem(trackKeyPrefix + 'load:' + whoami, JSON.stringify({\n        event: 'load',\n        bundleId: whoami,\n        bundlePath: 'target/classes/org/jenkins/ui/jsmodules/react/react-15-3-2.js',\n        at: loadTimestamp,\n        loadIdx: jenkinsCIGlobal._internal.bundleLoadIdx\n    }));\n}\n\nfunction ___$$$___doExport(namespace, moduleName, module) {\n    var exportedModuleId = (namespace !== undefined ? (namespace + ':' + moduleName) : moduleName);\n    try {\n        ___$$$___jsModules.requireModule(exportedModuleId);\n        // No exception => already exported. Shouldn't happen, but okay !!\n        return;\n    } catch (e) {\n        // Not exported yet, hence exception on require. Okay to export !!\n        ___$$$___jsModules.exportModule(namespace, moduleName, module);\n        if (window.sessionStorage) {\n            sessionStorage.setItem(trackKeyPrefix + 'export:' + exportedModuleId, JSON.stringify({\n                event: 'export',\n                moduleId: exportedModuleId,\n                at: loadTimestamp,\n                bundleId: whoami\n            }));\n        }\n    }\n}\n\n/*** Start Module Exec Function ***************************************/\nfunction ___$$$___exec(onExec) {\n    'use strict';\n\n//\n// NOTE: This file is generated and should NOT be added to source control.\n//\n\n___$$$___doExport('react', 'react@15.3.2', require('react'));\n___$$$___doExport('react', 'react@15.3.x', require('react'));\n___$$$___doExport('react', 'react@15.x', require('react'));\n___$$$___doExport('react', 'react@any', require('react'));\n\n    onExec();\n}\n/*** End Module Exec Function   ***************************************/\n\nfunction ___$$$___doExports() {\n    // Do exports, if any ....\n    \n\n}\n\nfunction ___$$$___doCSS() {\n}\n\nfunction ___$$$___doBundleInit() {\n    try {\n        ___$$$___exec(function() {\n            ___$$$___doExports();\n            ___$$$___doCSS();\n        });\n    } catch (e) {\n        console.error('Error initializing Jenkins JavaScript bundle \"react-15-3-2\"', e);\n    }\n}\n\nif (___$$$___requiredModuleMappings.length > 0) {\n    ___$$$___jsModules.importModule.apply(___$$$___jsModules.importModule, ___$$$___requiredModuleMappings)\n        .onFulfilled(function() {\n        ___$$$___doBundleInit();\n    });\n} else {\n    ___$$$___doBundleInit();\n}","deps":{"@jenkins-cd/js-modules":"node_modules/@jenkins-cd/js-modules/js/index.js","react":"node_modules/react/react.js"},"entry":true}]